#!/bin/sh -e

# Skript pro automatickou aktualizaci systemu.
# Priklada se do aktualizacniho balicku. Pri upgrade je spusten
# a provede prechod na novou verzi systemu.

# Nastaveni jmen stahovanych image souboru
ROOTFS_FILE=rootfs.ubifs
FDT_FILE=omr700.dtb
KERNEL_FILE=uImage-3.13-ts-armv7l
VERSION_FILE=/etc/system_version.txt

# Rozlozeni oddilu (viz /proc/mtd)
SYS0_FDT_LABEL="DTB_0"
SYS0_KERNEL_LABEL="Kernel_0"
SYS1_FDT_LABEL="DTB_1"
SYS1_KERNEL_LABEL="Kernel_1"
#SYS0_FDT_DEV=/dev/mtd2
#SYS0_KERNEL_DEV=/dev/mtd3
#SYS1_FDT_DEV=/dev/mtd4
#SYS1_KERNEL_DEV=/dev/mtd5

# Nastaveni cest
PATH=/usr/bin:/bin:/usr/sbin:/sbin
LOGDIR=/mnt/sysdata/syslog
LOGFILE=$LOGDIR/upgrade.log
FIFOFILE=/tmp/upgrade.fifo
WORKDIR=

# Pomocne promenne
FDT_SIZE=0
KERNEL_SIZE=0

# Nastaveni instalace
FLASH_FDT=NO
FLASH_KERNEL=NO
FLASH_RFS=NO
TIMESTAMP_FORMAT="date +"%Y-%m-%d_%H:%M:%S""    # Podle date -I (ISO format)
RFS_MTD="System"

trap "abort" EXIT INT TERM

# Funkce

# Vypise napovedu k pouziti do stdout. Jen pro debug
print_help () {
echo -e >&2 \
"usage: $0 [-acdkr]\n" \
"       -a      flash all (Kernel, Device tree, RFS)\n" \
"       -d      flash only the device tree\n" \
"       -k      flash only the kernel\n" \
"       -r      flash only RFS"
}

# Vrati cislo MTD oddilu podle nazvu
# $1 - Label
get_mtd_idx () {
        LABEL=$1

        IDX=$(cat /proc/mtd | grep "\"${LABEL}\"" | awk '{print $1}' | grep -o '[0-9]\+')
        if [ $? -ne 0 ]; then
                return 1
        fi

        echo ${IDX}
        return 0
}

# abort: spousti se pri EXITu
abort () {
	if [ "$UPGRADE_OK" = "YES" ]; then
		log "System install script exit OK"; report "249 100% 1"
	else
		log "Error -299: aborting!"
	fi

	cleanup
}

# Vypise argument do log souboru
log () {
	TIMESTAMP="[ $(eval $TIMESTAMP_FORMAT) ]"

	if [ -z "$1" ]; then
		echo $TIMESTAMP "Nothing to log..." # No parameters passed
	else
		echo $TIMESTAMP $1 >> $LOGFILE; echo $1
	fi
}

# Smazani docasnych souboru pri ukonceni
cleanup () {
	log "System install - cleanup"
	if [ -d $WORKDIR ]; then
		rm -r $WORKDIR/* || log "Cannot remove $WORKDIR contents!"
	fi

}

report () {
	if [ -z "$1" ]; then
		log "Warning: empty report message"
	else
		echo $1 > $FIFOFILE &
	fi
}

###############################################################################
#	START

# Kontrola slozek
if [ ! -d "$LOGDIR" ]; then
	mkdir $LOGDIR
fi

# Uvodni vypis
log "Starting System update script"; report "201 1% 1"

# Parsovani vstupnich parametru
while getopts ":acdkr:" opt;
do
	case "$opt" in
	# "-a" install all
	a)	log "Config: full system upgrade"
		FLASH_FDT=YES
		FLASH_KERNEL=YES
		FLASH_RFS=YES
		;;
	# "-d" flash only device tree
	d)	log "Config: Flashing only the device tree"
		FLASH_FDT=YES
		;;
	# "-k" flash only kernel
	k)	log "Config: Flashing only the kernel"
		FLASH_KERNEL=YES
		;;
	# "-r" flash only RFS
	r)	log "Config: Flashing only RFS"
		FLASH_RFS=YES
		;;
	# print help
	\?)	log "Error 251: unknown argument!"; report "-251"
		print_help
		exit 1;;
	esac
done

# Funguju v adresari package/files
cd ..
WORKDIR=$PWD	# Jak moc je tohle bezpecne?

# Zadny parametr - vypise napovedu
if [ "$FLASH_FDT" != "YES" ] && [ "$FLASH_FDT" != "YES" ] && \
	[ "$FLASH_FDT" != "YES" ] && [ "$FLASH_FDT" != "YES" ]; then
	log "Error -251: Wrong/No arguments!"; report "-251"
	print_help
	exit 1
fi
	
# Ktera kopie systemu prave bezi?
RFS0_TEST=`cat /proc/cmdline | grep -c ubi0:rootfs0`
RFS1_TEST=`cat /proc/cmdline | grep -c ubi0:rootfs1`

if [ "$RFS0_TEST" = "1" ] && [ "$RFS1_TEST" = "0" ]; then
        ACTIVE_SYS=0
elif [ "$RFS0_TEST" = "0" ] && [ "$RFS1_TEST" = 1 ]; then
        ACTIVE_SYS=1
else
        log "Error 252: determining active sys copy failed!" report "-252"
        exit 1
fi

# Vyber prepisovaneho systemu nastaveni cest
if [ "$ACTIVE_SYS" = "0" ]; then
        BACKUP_SYS=1
        FDT_DEV="/dev/mtd$(get_mtd_idx ${SYS1_FDT_LABEL})" \
        && KERNEL_DEV="/dev/mtd$(get_mtd_idx ${SYS1_KERNEL_LABEL})"
        if [ $? -ne 0 ]; then
            log "Error -264: Failed to get partition index!"; report "-264"
        fi
        ENV_FDT_SIZE=sys1_fdt_size
        ENV_KERNEL_SIZE=sys1_kernel_size
elif [ "$ACTIVE_SYS" = "1" ]; then
        BACKUP_SYS=0
        FDT_DEV="/dev/mtd$(get_mtd_idx ${SYS0_FDT_LABEL})" \
        && KERNEL_DEV="/dev/mtd$(get_mtd_idx ${SYS0_KERNEL_LABEL})"
        if [ $? -ne 0 ]; then
            log "Error -264: Failed to get partition index!"; report "-264"
        fi
        ENV_FDT_SIZE=sys0_fdt_size                                                                         
        ENV_KERNEL_SIZE=sys0_kernel_size
else
        log "Error -299: Error!"
        exit 1
fi

# echo "DEBUG. FDT_DEV: ${FDT_DEV} KERNEL_DEV: ${KERNEL_DEV}"

# Skok do pracovniho adresare
cd $WORKDIR/files

# Flashovani Device Tree
if [ "$FLASH_FDT" = "YES" ]; then
	if [ ! -f $FDT_FILE ]; then
		log "Error -253: Device tree image not found!"; report "-253"
		exit 1
	fi
	log "Flashing device tree"; report "202 10% 0"
	SIZE=`wc -c $FDT_FILE | awk '{print $1}'`
	FDT_SIZE=$( printf "0x%x" $SIZE )
	log "FDT size: $FDT_SIZE"

	# Mazani NAND
	if ! flash_erase $FDT_DEV 0 0; then
        	log "Error 254: Flash_erase dtb partition failed!"; report "-254"
	        exit 1
	fi
	# Flahovani
	if ! nandwrite -p $FDT_DEV $FDT_FILE; then
        	log "Error 255: FDT nandwrite failed!"; report "-255"
	        exit 1
	fi
	# Aktualizace u-boot promennych
	if ! fw_setenv $ENV_FDT_SIZE $FDT_SIZE; then
        	log "Error 256: fw_setenv fdt_size failed!"; report "-256"
	        exit 1
	fi
fi

# Flashovani kernelu
if [ "$FLASH_KERNEL" = "YES" ]; then
        if [ ! -f $KERNEL_FILE ]; then
                log "Error 257: Kernel image not found!"; report "-257"
                exit 1
        fi
	log "Flashing kernel"; report "203 20% 0"
	SIZE=`wc -c $KERNEL_FILE | awk '{print $1}'`
	KERNEL_SIZE=$( printf "0x%x" $SIZE )
	log "Kernel size: $KERNEL_SIZE"

        # Mazani NAND
        if ! flash_erase $KERNEL_DEV 0 0; then
                log "Error 258: Cannot erase kernel partition"; report "-258"
                exit 1
        fi
        # Flahovani
        if ! nandwrite -p $KERNEL_DEV $KERNEL_FILE; then
                log "Error 259: Cannot flash kernel image!"; report "-259"
                exit 1
        fi
        # Aktualizace u-boot promennych
        if ! fw_setenv $ENV_KERNEL_SIZE $KERNEL_SIZE; then
                log "Error 260: Cannot set new kernel size"; report "-260"
                exit 1
        fi
        # Oprava nastaveni RFS MTD
        if [ "$(fw_printenv -n rfs_mtd)" != "${RFS_MTD}" ]; then
                if ! fw_setenv rfs_mtd ${RFS_MTD}; then
                        log "Error 264: Cannot set RFS MTD label"; report "-264"
                        exit 1
                fi
        fi
fi

# Flashovani RFS
if [ "$FLASH_RFS" = "YES" ]; then
	if [ ! -f $RFS_FILE ]; then
		log "Error 261: RFS image not found!"; report "-261"
		exit 1
	fi
	log "Flashing new filesystem"; report "204 40% 0"
	# Zkontroluju, jestli neni nahodou mountnutej
	if df | grep -q /dev/ubi0_$BACKUP_SYS; then
        	log "Warn: RFS already mounted, attempting umount!"
        	if ! umount /dev/ubi0_$BACKUP_SYS; then
                	log "Error 262: RFS umount failed!"; report "-262"
               		exit 1
        	fi
	fi
	# Flashovani logickeho oddilu
	if ! ubiupdatevol /dev/ubi0_$BACKUP_SYS $ROOTFS_FILE; then
        	log "Error 263: RFS flashing failed!"; report "-263"
        	exit 1
	fi
fi

# Zakonceni
# Jestli to dobehlo az sem, melo by bejt vsechno OK
UPGRADE_OK=YES

# Cisteni - v exitu

# Konec
exit 0

